@@ -3,7 +3,6 @@
 mod ast;
 mod auth_tokens;
 mod checksum;
-mod colors;
 mod config_file;
 mod deno_dir;
 mod diagnostics;
@@ -18,12 +17,10 @@ mod fmt_errors;
 mod fs_util;
 mod http_cache;
 mod http_util;
-mod import_map;
 mod info;
 mod lockfile;
 mod logger;
 mod lsp;
-mod media_type;
 mod module_graph;
 mod module_loader;
 mod ops;
@@ -40,17 +37,30 @@ mod version;
 
 use crate::file_fetcher::File;
 use crate::file_watcher::ResolutionResult;
+use crate::flags::BundleFlags;
+use crate::flags::CacheFlags;
+use crate::flags::CompileFlags;
+use crate::flags::CompletionsFlags;
+use crate::flags::CoverageFlags;
 use crate::flags::DenoSubcommand;
+use crate::flags::DocFlags;
+use crate::flags::EvalFlags;
 use crate::flags::Flags;
+use crate::flags::FmtFlags;
+use crate::flags::InfoFlags;
+use crate::flags::InstallFlags;
+use crate::flags::LintFlags;
+use crate::flags::ReplFlags;
+use crate::flags::RunFlags;
+use crate::flags::TestFlags;
+use crate::flags::UpgradeFlags;
 use crate::fmt_errors::PrettyJsError;
-use crate::media_type::MediaType;
-use crate::module_graph::GraphBuilder;
-use crate::module_graph::Module;
 use crate::module_loader::CliModuleLoader;
 use crate::program_state::ProgramState;
 use crate::source_maps::apply_source_map;
 use crate::specifier_handler::FetchHandler;
 use crate::tools::installer::infer_name_from_url;
+use deno_ast::MediaType;
 use deno_core::error::generic_error;
 use deno_core::error::AnyError;
 use deno_core::futures::future::FutureExt;
@@ -63,6 +73,7 @@ use deno_core::serde_json::json;
 use deno_core::v8_set_flags;
 use deno_core::JsRuntime;
 use deno_core::ModuleSpecifier;
+use deno_runtime::colors;
 use deno_runtime::ops::worker_host::CreateWebWorkerCb;
 use deno_runtime::permissions::Permissions;
 use deno_runtime::web_worker::WebWorker;
@@ -71,17 +82,14 @@ use deno_runtime::worker::MainWorker;
 use deno_runtime::worker::WorkerOptions;
 use log::debug;
 use log::info;
-use std::collections::HashSet;
 use std::env;
 use std::io::Read;
 use std::io::Write;
 use std::iter::once;
-use std::num::NonZeroUsize;
 use std::path::PathBuf;
 use std::pin::Pin;
 use std::rc::Rc;
 use std::sync::Arc;
-use tools::test_runner;
 
 fn create_web_worker_callback(
   program_state: Arc<ProgramState>,
@@ -362,8 +370,6 @@ pub fn get_types(unstable: bool) -> String {
 
   if unstable {
     types.push(crate::tsc::UNSTABLE_NS_LIB);
-    types.push(crate::tsc::DENO_NET_UNSTABLE_LIB);
-    types.push(crate::tsc::DENO_HTTP_UNSTABLE_LIB);
   }
 
   types.join("\n")
@@ -371,21 +377,20 @@ pub fn get_types(unstable: bool) -> String {
 
 async fn compile_command(
   flags: Flags,
-  source_file: String,
-  output: Option<PathBuf>,
-  args: Vec<String>,
-  target: Option<String>,
+  compile_flags: CompileFlags,
 ) -> Result<(), AnyError> {
   let debug = flags.log_level == Some(log::Level::Debug);
 
-  let run_flags =
-    tools::standalone::compile_to_runtime_flags(flags.clone(), args)?;
+  let run_flags = tools::standalone::compile_to_runtime_flags(
+    flags.clone(),
+    compile_flags.args,
+  )?;
 
-  let module_specifier = resolve_url_or_path(&source_file)?;
+  let module_specifier = resolve_url_or_path(&compile_flags.source_file)?;
   let program_state = ProgramState::build(flags.clone()).await?;
   let deno_dir = &program_state.dir;
 
-  let output = output.or_else(|| {
+  let output = compile_flags.output.or_else(|| {
     infer_name_from_url(&module_specifier).map(PathBuf::from)
   }).ok_or_else(|| generic_error(
     "An executable name was not provided. One could not be inferred from the URL. Aborting.",
@@ -414,7 +419,8 @@ async fn compile_command(
 
   // Select base binary based on target
   let original_binary =
-    tools::standalone::get_base_binary(deno_dir, target.clone()).await?;
+    tools::standalone::get_base_binary(deno_dir, compile_flags.target.clone())
+      .await?;
 
   let final_bin = tools::standalone::create_standalone_binary(
     original_binary,
@@ -424,20 +430,23 @@ async fn compile_command(
 
   info!("{} {}", colors::green("Emit"), output.display());
 
-  tools::standalone::write_standalone_binary(output.clone(), target, final_bin)
-    .await?;
+  tools::standalone::write_standalone_binary(
+    output.clone(),
+    compile_flags.target,
+    final_bin,
+  )
+  .await?;
 
   Ok(())
 }
 
 async fn info_command(
   flags: Flags,
-  maybe_specifier: Option<String>,
-  json: bool,
+  info_flags: InfoFlags,
 ) -> Result<(), AnyError> {
   let location = flags.location.clone();
   let program_state = ProgramState::build(flags).await?;
-  if let Some(specifier) = maybe_specifier {
+  if let Some(specifier) = info_flags.file {
     let specifier = resolve_url_or_path(&specifier)?;
     let handler = Arc::new(Mutex::new(specifier_handler::FetchHandler::new(
       &program_state,
@@ -458,7 +467,7 @@ async fn info_command(
     let graph = builder.get_graph();
     let info = graph.info()?;
 
-    if json {
+    if info_flags.json {
       write_json_to_stdout(&json!(info))
     } else {
       write_to_stdout_ignore_sigpipe(info.to_string().as_bytes())
@@ -466,53 +475,71 @@ async fn info_command(
     }
   } else {
     // If it was just "deno info" print location of caches and exit
-    print_cache_info(&program_state, json, location)
+    print_cache_info(&program_state, info_flags.json, location)
   }
 }
 
 async fn install_command(
   flags: Flags,
-  module_url: String,
-  args: Vec<String>,
-  name: Option<String>,
-  root: Option<PathBuf>,
-  force: bool,
+  install_flags: InstallFlags,
 ) -> Result<(), AnyError> {
   let mut preload_flags = flags.clone();
   preload_flags.inspect = None;
   preload_flags.inspect_brk = None;
   let permissions = Permissions::from_options(&preload_flags.clone().into());
   let program_state = ProgramState::build(preload_flags).await?;
-  let main_module = resolve_url_or_path(&module_url)?;
+  let main_module = resolve_url_or_path(&install_flags.module_url)?;
   let mut worker =
     create_main_worker(&program_state, main_module.clone(), permissions, None);
   // First, fetch and compile the module; this step ensures that the module exists.
   worker.preload_module(&main_module).await?;
-  tools::installer::install(flags, &module_url, args, name, root, force)
+  tools::installer::install(
+    flags,
+    &install_flags.module_url,
+    install_flags.args,
+    install_flags.name,
+    install_flags.root,
+    install_flags.force,
+  )
 }
 
 async fn lsp_command() -> Result<(), AnyError> {
   lsp::start().await
 }
 
+#[allow(clippy::too_many_arguments)]
 async fn lint_command(
-  _flags: Flags,
-  files: Vec<PathBuf>,
-  list_rules: bool,
-  ignore: Vec<PathBuf>,
-  json: bool,
+  flags: Flags,
+  lint_flags: LintFlags,
 ) -> Result<(), AnyError> {
-  if list_rules {
-    tools::lint::print_rules_list(json);
+  if lint_flags.rules {
+    tools::lint::print_rules_list(lint_flags.json);
     return Ok(());
   }
 
-  tools::lint::lint_files(files, ignore, json).await
+  let program_state = ProgramState::build(flags.clone()).await?;
+  let maybe_lint_config =
+    if let Some(config_file) = &program_state.maybe_config_file {
+      config_file.to_lint_config()?
+    } else {
+      None
+    };
+
+  tools::lint::lint_files(
+    maybe_lint_config,
+    lint_flags.rules_tags,
+    lint_flags.rules_include,
+    lint_flags.rules_exclude,
+    lint_flags.files,
+    lint_flags.ignore,
+    lint_flags.json,
+  )
+  .await
 }
 
 async fn cache_command(
   flags: Flags,
-  files: Vec<String>,
+  cache_flags: CacheFlags,
 ) -> Result<(), AnyError> {
   let lib = if flags.unstable {
     module_graph::TypeLib::UnstableDenoWindow
@@ -521,7 +548,7 @@ async fn cache_command(
   };
   let program_state = ProgramState::build(flags).await?;
 
-  for file in files {
+  for file in cache_flags.files {
     let specifier = resolve_url_or_path(&file)?;
     program_state
       .prepare_module_load(
@@ -540,9 +567,7 @@ async fn cache_command(
 
 async fn eval_command(
   flags: Flags,
-  code: String,
-  ext: String,
-  print: bool,
+  eval_flags: EvalFlags,
 ) -> Result<(), AnyError> {
   // Force TypeScript compile.
   let main_module = resolve_url_or_path("./$deno$eval.ts").unwrap();
@@ -551,27 +576,28 @@ async fn eval_command(
   let mut worker =
     create_main_worker(&program_state, main_module.clone(), permissions, None);
   // Create a dummy source file.
-  let source_code = if print {
-    format!("console.log({})", code)
+  let source_code = if eval_flags.print {
+    format!("console.log({})", eval_flags.code)
   } else {
-    code
+    eval_flags.code
   }
   .into_bytes();
 
   let file = File {
     local: main_module.clone().to_file_path().unwrap(),
     maybe_types: None,
-    media_type: if ext.as_str() == "ts" {
+    media_type: if eval_flags.ext.as_str() == "ts" {
       MediaType::TypeScript
-    } else if ext.as_str() == "tsx" {
+    } else if eval_flags.ext.as_str() == "tsx" {
       MediaType::Tsx
-    } else if ext.as_str() == "js" {
+    } else if eval_flags.ext.as_str() == "js" {
       MediaType::JavaScript
     } else {
       MediaType::Jsx
     },
-    source: String::from_utf8(source_code)?,
+    source: Arc::new(String::from_utf8(source_code)?),
     specifier: main_module.clone(),
+    maybe_headers: None,
   };
 
   // Save our fake file into file fetcher cache
@@ -666,15 +692,14 @@ fn bundle_module_graph(
 
 async fn bundle_command(
   flags: Flags,
-  source_file: String,
-  out_file: Option<PathBuf>,
+  bundle_flags: BundleFlags,
 ) -> Result<(), AnyError> {
   let debug = flags.log_level == Some(log::Level::Debug);
 
   let resolver = |_| {
     let flags = flags.clone();
-    let source_file1 = source_file.clone();
-    let source_file2 = source_file.clone();
+    let source_file1 = bundle_flags.source_file.clone();
+    let source_file2 = bundle_flags.source_file.clone();
     async move {
       let module_specifier = resolve_url_or_path(&source_file1)?;
 
@@ -720,7 +745,7 @@ async fn bundle_command(
     module_graph::Graph,
   )| {
     let flags = flags.clone();
-    let out_file = out_file.clone();
+    let out_file = bundle_flags.out_file.clone();
     async move {
       info!("{} {}", colors::green("Bundle"), module_graph.info()?.root);
 
@@ -764,33 +789,42 @@ async fn bundle_command(
 
 async fn doc_command(
   flags: Flags,
-  source_file: Option<String>,
-  json: bool,
-  maybe_filter: Option<String>,
-  private: bool,
+  doc_flags: DocFlags,
 ) -> Result<(), AnyError> {
-  tools::doc::print_docs(flags, source_file, json, maybe_filter, private).await
+  tools::doc::print_docs(
+    flags,
+    doc_flags.source_file,
+    doc_flags.json,
+    doc_flags.filter,
+    doc_flags.private,
+  )
+  .await
 }
 
 async fn format_command(
   flags: Flags,
-  args: Vec<PathBuf>,
-  ignore: Vec<PathBuf>,
-  check: bool,
-  ext: String,
+  fmt_flags: FmtFlags,
 ) -> Result<(), AnyError> {
-  if args.len() == 1 && args[0].to_string_lossy() == "-" {
-    return tools::fmt::format_stdin(check, ext);
+  let program_state = ProgramState::build(flags.clone()).await?;
+  let maybe_fmt_config =
+    if let Some(config_file) = &program_state.maybe_config_file {
+      config_file.to_fmt_config()?
+    } else {
+      None
+    };
+
+  if fmt_flags.files.len() == 1 && fmt_flags.files[0].to_string_lossy() == "-" {
+    return tools::fmt::format_stdin(
+      fmt_flags,
+      maybe_fmt_config.map(|c| c.options).unwrap_or_default(),
+    );
   }
 
-  tools::fmt::format(args, ignore, check, flags.watch).await?;
+  tools::fmt::format(fmt_flags, flags.watch, maybe_fmt_config).await?;
   Ok(())
 }
 
-async fn run_repl(
-  flags: Flags,
-  maybe_eval: Option<String>,
-) -> Result<(), AnyError> {
+async fn run_repl(flags: Flags, repl_flags: ReplFlags) -> Result<(), AnyError> {
   let main_module = resolve_url_or_path("./$deno$repl.ts").unwrap();
   let permissions = Permissions::from_options(&flags.clone().into());
   let program_state = ProgramState::build(flags).await?;
@@ -798,7 +832,7 @@ async fn run_repl(
     create_main_worker(&program_state, main_module.clone(), permissions, None);
   worker.run_event_loop(false).await?;
 
-  tools::repl::run(&program_state, worker, maybe_eval).await
+  tools::repl::run(&program_state, worker, repl_flags.eval).await
 }
 
 async fn run_from_stdin(flags: Flags) -> Result<(), AnyError> {
@@ -819,8 +853,9 @@ async fn run_from_stdin(flags: Flags) -> Result<(), AnyError> {
     local: main_module.clone().to_file_path().unwrap(),
     maybe_types: None,
     media_type: MediaType::TypeScript,
-    source: String::from_utf8(source)?,
+    source: Arc::new(String::from_utf8(source)?),
     specifier: main_module.clone(),
+    maybe_headers: None,
   };
   // Save our fake file into file fetcher cache
   // to allow module access by TS compiler
@@ -892,29 +927,81 @@ async fn run_with_watch(flags: Flags, script: String) -> Result<(), AnyError> {
     })
   };
 
+  /// The FileWatcherModuleExecutor provides module execution with safe dispatching of life-cycle events by tracking the
+  /// state of any pending events and emitting accordingly on drop in the case of a future
+  /// cancellation.
+  struct FileWatcherModuleExecutor {
+    worker: MainWorker,
+    pending_unload: bool,
+  }
+
+  impl FileWatcherModuleExecutor {
+    pub fn new(worker: MainWorker) -> FileWatcherModuleExecutor {
+      FileWatcherModuleExecutor {
+        worker,
+        pending_unload: false,
+      }
+    }
+
+    /// Execute the given main module emitting load and unload events before and after execution
+    /// respectively.
+    pub async fn execute(
+      &mut self,
+      main_module: &ModuleSpecifier,
+    ) -> Result<(), AnyError> {
+      self.worker.execute_module(main_module).await?;
+      self.worker.execute_script(
+        &located_script_name!(),
+        "window.dispatchEvent(new Event('load'))",
+      )?;
+      self.pending_unload = true;
+
+      let result = self.worker.run_event_loop(false).await;
+      self.pending_unload = false;
+
+      if let Err(err) = result {
+        return Err(err);
+      }
+
+      self.worker.execute_script(
+        &located_script_name!(),
+        "window.dispatchEvent(new Event('unload'))",
+      )?;
+
+      Ok(())
+    }
+  }
+
+  impl Drop for FileWatcherModuleExecutor {
+    fn drop(&mut self) {
+      if self.pending_unload {
+        self
+          .worker
+          .execute_script(
+            &located_script_name!(),
+            "window.dispatchEvent(new Event('unload'))",
+          )
+          .unwrap();
+      }
+    }
+  }
+
   let operation =
     |(program_state, main_module): (Arc<ProgramState>, ModuleSpecifier)| {
       let flags = flags.clone();
       let permissions = Permissions::from_options(&flags.into());
       async move {
-        let main_module = main_module.clone();
-        let mut worker = create_main_worker(
+        // We make use an module executor guard to ensure that unload is always fired when an
+        // operation is called.
+        let mut executor = FileWatcherModuleExecutor::new(create_main_worker(
           &program_state,
           main_module.clone(),
           permissions,
           None,
-        );
-        debug!("main_module {}", main_module);
-        worker.execute_module(&main_module).await?;
-        worker.execute_script(
-          &located_script_name!(),
-          "window.dispatchEvent(new Event('load'))",
-        )?;
-        worker.run_event_loop(false).await?;
-        worker.execute_script(
-          &located_script_name!(),
-          "window.dispatchEvent(new Event('unload'))",
-        )?;
+        ));
+
+        executor.execute(&main_module).await?;
+
         Ok(())
       }
     };
@@ -922,17 +1009,20 @@ async fn run_with_watch(flags: Flags, script: String) -> Result<(), AnyError> {
   file_watcher::watch_func(resolver, operation, "Process").await
 }
 
-async fn run_command(flags: Flags, script: String) -> Result<(), AnyError> {
+async fn run_command(
+  flags: Flags,
+  run_flags: RunFlags,
+) -> Result<(), AnyError> {
   // Read script content from stdin
-  if script == "-" {
+  if run_flags.script == "-" {
     return run_from_stdin(flags).await;
   }
 
   if flags.watch {
-    return run_with_watch(flags, script).await;
+    return run_with_watch(flags, run_flags.script).await;
   }
 
-  let main_module = resolve_url_or_path(&script)?;
+  let main_module = resolve_url_or_path(&run_flags.script)?;
   let program_state = ProgramState::build(flags.clone()).await?;
   let permissions = Permissions::from_options(&flags.clone().into());
   let mut worker =
@@ -977,38 +1067,26 @@ async fn run_command(flags: Flags, script: String) -> Result<(), AnyError> {
 
 async fn coverage_command(
   flags: Flags,
-  files: Vec<PathBuf>,
-  ignore: Vec<PathBuf>,
-  include: Vec<String>,
-  exclude: Vec<String>,
-  lcov: bool,
+  coverage_flags: CoverageFlags,
 ) -> Result<(), AnyError> {
-  if files.is_empty() {
+  if coverage_flags.files.is_empty() {
     return Err(generic_error("No matching coverage profiles found"));
   }
 
   tools::coverage::cover_files(
     flags.clone(),
-    files,
-    ignore,
-    include,
-    exclude,
-    lcov,
+    coverage_flags.files,
+    coverage_flags.ignore,
+    coverage_flags.include,
+    coverage_flags.exclude,
+    coverage_flags.lcov,
   )
   .await
 }
 
-#[allow(clippy::too_many_arguments)]
 async fn test_command(
   flags: Flags,
-  include: Option<Vec<String>>,
-  no_run: bool,
-  doc: bool,
-  fail_fast: Option<NonZeroUsize>,
-  allow_none: bool,
-  filter: Option<String>,
-  shuffle: Option<u64>,
-  concurrent_jobs: NonZeroUsize,
+  test_flags: TestFlags,
 ) -> Result<(), AnyError> {
   if let Some(ref coverage_dir) = flags.coverage_dir {
     std::fs::create_dir_all(&coverage_dir)?;
@@ -1018,246 +1096,37 @@ async fn test_command(
     );
   }
 
-  // TODO(caspervonb) move this chunk into tools::test_runner.
-
-  let program_state = ProgramState::build(flags.clone()).await?;
-
-  let include = include.unwrap_or_else(|| vec![".".to_string()]);
-
-  let permissions = Permissions::from_options(&flags.clone().into());
-  let lib = if flags.unstable {
-    module_graph::TypeLib::UnstableDenoWindow
-  } else {
-    module_graph::TypeLib::DenoWindow
-  };
-
   if flags.watch {
-    let handler = Arc::new(Mutex::new(FetchHandler::new(
-      &program_state,
-      Permissions::allow_all(),
-      Permissions::allow_all(),
-    )?));
-
-    let paths_to_watch: Vec<_> = include.iter().map(PathBuf::from).collect();
-
-    // TODO(caspervonb) clean this up.
-    let resolver = |changed: Option<Vec<PathBuf>>| {
-      let test_modules_result = if doc {
-        fs_util::collect_specifiers(
-          include.clone(),
-          fs_util::is_supported_test_ext,
-        )
-      } else {
-        fs_util::collect_specifiers(
-          include.clone(),
-          fs_util::is_supported_test_path,
-        )
-      };
-
-      let paths_to_watch = paths_to_watch.clone();
-      let paths_to_watch_clone = paths_to_watch.clone();
-
-      let handler = handler.clone();
-      let program_state = program_state.clone();
-      let files_changed = changed.is_some();
-      async move {
-        let test_modules = test_modules_result?;
-
-        let mut paths_to_watch = paths_to_watch_clone;
-        let mut modules_to_reload = if files_changed {
-          Vec::new()
-        } else {
-          test_modules
-            .iter()
-            .filter_map(|url| deno_core::resolve_url(url.as_str()).ok())
-            .collect()
-        };
-
-        let mut builder = GraphBuilder::new(
-          handler,
-          program_state.maybe_import_map.clone(),
-          program_state.lockfile.clone(),
-        );
-        for specifier in test_modules.iter() {
-          builder.add(specifier, false).await?;
-        }
-        builder
-          .analyze_config_file(&program_state.maybe_config_file)
-          .await?;
-        let graph = builder.get_graph();
-
-        for specifier in test_modules {
-          fn get_dependencies<'a>(
-            graph: &'a module_graph::Graph,
-            module: &'a Module,
-            // This needs to be accessible to skip getting dependencies if they're already there,
-            // otherwise this will cause a stack overflow with circular dependencies
-            output: &mut HashSet<&'a ModuleSpecifier>,
-          ) -> Result<(), AnyError> {
-            for dep in module.dependencies.values() {
-              if let Some(specifier) = &dep.maybe_code {
-                if !output.contains(specifier) {
-                  output.insert(specifier);
-
-                  get_dependencies(
-                    graph,
-                    graph.get_specifier(specifier)?,
-                    output,
-                  )?;
-                }
-              }
-              if let Some(specifier) = &dep.maybe_type {
-                if !output.contains(specifier) {
-                  output.insert(specifier);
-
-                  get_dependencies(
-                    graph,
-                    graph.get_specifier(specifier)?,
-                    output,
-                  )?;
-                }
-              }
-            }
-
-            Ok(())
-          }
-
-          // This test module and all it's dependencies
-          let mut modules = HashSet::new();
-          modules.insert(&specifier);
-          get_dependencies(
-            &graph,
-            graph.get_specifier(&specifier)?,
-            &mut modules,
-          )?;
-
-          paths_to_watch.extend(
-            modules
-              .iter()
-              .filter_map(|specifier| specifier.to_file_path().ok()),
-          );
-
-          if let Some(changed) = &changed {
-            for path in changed.iter().filter_map(|path| {
-              deno_core::resolve_url_or_path(&path.to_string_lossy()).ok()
-            }) {
-              if modules.contains(&&path) {
-                modules_to_reload.push(specifier);
-                break;
-              }
-            }
-          }
-        }
-
-        Ok((paths_to_watch, modules_to_reload))
-      }
-      .map(move |result| {
-        if files_changed
-          && matches!(result, Ok((_, ref modules)) if modules.is_empty())
-        {
-          ResolutionResult::Ignore
-        } else {
-          match result {
-            Ok((paths_to_watch, modules_to_reload)) => {
-              ResolutionResult::Restart {
-                paths_to_watch,
-                result: Ok(modules_to_reload),
-              }
-            }
-            Err(e) => ResolutionResult::Restart {
-              paths_to_watch,
-              result: Err(e),
-            },
-          }
-        }
-      })
-    };
-
-    let operation = |modules_to_reload: Vec<ModuleSpecifier>| {
-      let filter = filter.clone();
-      let include = include.clone();
-      let lib = lib.clone();
-      let permissions = permissions.clone();
-      let program_state = program_state.clone();
-
-      async move {
-        let doc_modules = if doc {
-          fs_util::collect_specifiers(
-            include.clone(),
-            fs_util::is_supported_test_ext,
-          )?
-        } else {
-          Vec::new()
-        };
-
-        let doc_modules_to_reload = doc_modules
-          .iter()
-          .filter(|specifier| modules_to_reload.contains(specifier))
-          .cloned()
-          .collect();
-
-        let test_modules = fs_util::collect_specifiers(
-          include.clone(),
-          fs_util::is_supported_test_path,
-        )?;
-
-        let test_modules_to_reload = test_modules
-          .iter()
-          .filter(|specifier| modules_to_reload.contains(specifier))
-          .cloned()
-          .collect();
-
-        test_runner::run_tests(
-          program_state.clone(),
-          permissions.clone(),
-          lib.clone(),
-          doc_modules_to_reload,
-          test_modules_to_reload,
-          no_run,
-          fail_fast,
-          true,
-          filter.clone(),
-          shuffle,
-          concurrent_jobs,
-        )
-        .await?;
-
-        Ok(())
-      }
-    };
-
-    file_watcher::watch_func(resolver, operation, "Test").await?;
-  } else {
-    let doc_modules = if doc {
-      fs_util::collect_specifiers(
-        include.clone(),
-        fs_util::is_supported_test_ext,
-      )?
-    } else {
-      Vec::new()
-    };
-
-    let test_modules = fs_util::collect_specifiers(
-      include.clone(),
-      fs_util::is_supported_test_path,
-    )?;
-
-    test_runner::run_tests(
-      program_state.clone(),
-      permissions,
-      lib,
-      doc_modules,
-      test_modules,
-      no_run,
-      fail_fast,
-      allow_none,
-      filter,
-      shuffle,
-      concurrent_jobs,
+    tools::test::run_tests_with_watch(
+      flags,
+      test_flags.include,
+      test_flags.ignore,
+      test_flags.doc,
+      test_flags.no_run,
+      test_flags.fail_fast,
+      test_flags.filter,
+      test_flags.shuffle,
+      test_flags.concurrent_jobs,
     )
     .await?;
+
+    return Ok(());
   }
 
+  tools::test::run_tests(
+    flags,
+    test_flags.include,
+    test_flags.ignore,
+    test_flags.doc,
+    test_flags.no_run,
+    test_flags.fail_fast,
+    test_flags.allow_none,
+    test_flags.filter,
+    test_flags.shuffle,
+    test_flags.concurrent_jobs,
+  )
+  .await?;
+
   Ok(())
 }
 
@@ -1289,87 +1158,47 @@ fn get_subcommand(
   flags: Flags,
 ) -> Pin<Box<dyn Future<Output = Result<(), AnyError>>>> {
   match flags.clone().subcommand {
-    DenoSubcommand::Bundle {
-      source_file,
-      out_file,
-    } => bundle_command(flags, source_file, out_file).boxed_local(),
-    DenoSubcommand::Doc {
-      source_file,
-      json,
-      filter,
-      private,
-    } => doc_command(flags, source_file, json, filter, private).boxed_local(),
-    DenoSubcommand::Eval { print, code, ext } => {
-      eval_command(flags, code, ext, print).boxed_local()
+    DenoSubcommand::Bundle(bundle_flags) => {
+      bundle_command(flags, bundle_flags).boxed_local()
     }
-    DenoSubcommand::Cache { files } => {
-      cache_command(flags, files).boxed_local()
+    DenoSubcommand::Doc(doc_flags) => {
+      doc_command(flags, doc_flags).boxed_local()
     }
-    DenoSubcommand::Compile {
-      source_file,
-      output,
-      args,
-      target,
-    } => {
-      compile_command(flags, source_file, output, args, target).boxed_local()
+    DenoSubcommand::Eval(eval_flags) => {
+      eval_command(flags, eval_flags).boxed_local()
     }
-    DenoSubcommand::Coverage {
-      files,
-      ignore,
-      include,
-      exclude,
-      lcov,
-    } => coverage_command(flags, files, ignore, include, exclude, lcov)
-      .boxed_local(),
-    DenoSubcommand::Fmt {
-      check,
-      files,
-      ignore,
-      ext,
-    } => format_command(flags, files, ignore, check, ext).boxed_local(),
-    DenoSubcommand::Info { file, json } => {
-      info_command(flags, file, json).boxed_local()
+    DenoSubcommand::Cache(cache_flags) => {
+      cache_command(flags, cache_flags).boxed_local()
     }
-    DenoSubcommand::Install {
-      module_url,
-      args,
-      name,
-      root,
-      force,
-    } => {
-      install_command(flags, module_url, args, name, root, force).boxed_local()
+    DenoSubcommand::Compile(compile_flags) => {
+      compile_command(flags, compile_flags).boxed_local()
+    }
+    DenoSubcommand::Coverage(coverage_flags) => {
+      coverage_command(flags, coverage_flags).boxed_local()
+    }
+    DenoSubcommand::Fmt(fmt_flags) => {
+      format_command(flags, fmt_flags).boxed_local()
+    }
+    DenoSubcommand::Info(info_flags) => {
+      info_command(flags, info_flags).boxed_local()
+    }
+    DenoSubcommand::Install(install_flags) => {
+      install_command(flags, install_flags).boxed_local()
     }
     DenoSubcommand::Lsp => lsp_command().boxed_local(),
-    DenoSubcommand::Lint {
-      files,
-      rules,
-      ignore,
-      json,
-    } => lint_command(flags, files, rules, ignore, json).boxed_local(),
-    DenoSubcommand::Repl { eval } => run_repl(flags, eval).boxed_local(),
-    DenoSubcommand::Run { script } => run_command(flags, script).boxed_local(),
-    DenoSubcommand::Test {
-      no_run,
-      doc,
-      fail_fast,
-      include,
-      allow_none,
-      filter,
-      shuffle,
-      concurrent_jobs,
-    } => test_command(
-      flags,
-      include,
-      no_run,
-      doc,
-      fail_fast,
-      allow_none,
-      filter,
-      shuffle,
-      concurrent_jobs,
-    )
-    .boxed_local(),
-    DenoSubcommand::Completions { buf } => {
+    DenoSubcommand::Lint(lint_flags) => {
+      lint_command(flags, lint_flags).boxed_local()
+    }
+    DenoSubcommand::Repl(repl_flags) => {
+      run_repl(flags, repl_flags).boxed_local()
+    }
+    DenoSubcommand::Run(run_flags) => {
+      run_command(flags, run_flags).boxed_local()
+    }
+    DenoSubcommand::Test(test_flags) => {
+      test_command(flags, test_flags).boxed_local()
+    }
+    DenoSubcommand::Completions(CompletionsFlags { buf }) => {
       if let Err(e) = write_to_stdout_ignore_sigpipe(&buf) {
         eprintln!("{}", e);
         std::process::exit(1);
@@ -1384,20 +1213,33 @@ fn get_subcommand(
       }
       std::process::exit(0);
     }
-    DenoSubcommand::Upgrade {
-      force,
-      dry_run,
-      canary,
-      version,
-      output,
-      ca_file,
-    } => tools::upgrade::upgrade_command(
-      dry_run, force, canary, version, output, ca_file,
-    )
-    .boxed_local(),
+    DenoSubcommand::Upgrade(upgrade_flags) => {
+      let UpgradeFlags {
+        force,
+        dry_run,
+        canary,
+        version,
+        output,
+        ca_file,
+      } = upgrade_flags;
+      tools::upgrade::upgrade_command(
+        dry_run, force, canary, version, output, ca_file,
+      )
+      .boxed_local()
+    }
   }
 }
 
+fn setup_exit_process_panic_hook() {
+  // tokio does not exit the process when a task panics, so we
+  // define a custom panic hook to implement this behaviour
+  let orig_hook = std::panic::take_hook();
+  std::panic::set_hook(Box::new(move |panic_info| {
+    orig_hook(panic_info);
+    std::process::exit(1);
+  }));
+}
+
 fn unwrap_or_exit<T>(result: Result<T, AnyError>) -> T {
   match result {
     Ok(value) => value,
@@ -1409,6 +1251,7 @@ fn unwrap_or_exit<T>(result: Result<T, AnyError>) -> T {
 }
 
 pub fn main() {
+  setup_exit_process_panic_hook();
   #[cfg(windows)]
   colors::enable_ansi(); // For Windows 10
   unix_util::raise_fd_limit();
