@@ -8,7 +8,7 @@ import {
   unitTest,
 } from "./test_util.ts";
 
-function assertSameContent(files: Deno.DirEntry[]): void {
+function assertSameContent(files: Deno.DirEntry[]) {
   let counter = 0;
 
   for (const entry of files) {
@@ -21,59 +21,55 @@ function assertSameContent(files: Deno.DirEntry[]): void {
   assertEquals(counter, 1);
 }
 
-unitTest({ perms: { read: true } }, function readDirSyncSuccess(): void {
-  const files = [...Deno.readDirSync("cli/tests/")];
+unitTest({ perms: { read: true } }, function readDirSyncSuccess() {
+  const files = [...Deno.readDirSync("cli/tests/testdata")];
   assertSameContent(files);
 });
 
-unitTest({ perms: { read: true } }, function readDirSyncWithUrl(): void {
-  const files = [...Deno.readDirSync(pathToAbsoluteFileUrl("cli/tests"))];
+unitTest({ perms: { read: true } }, function readDirSyncWithUrl() {
+  const files = [
+    ...Deno.readDirSync(pathToAbsoluteFileUrl("cli/tests/testdata")),
+  ];
   assertSameContent(files);
 });
 
-unitTest({ perms: { read: false } }, function readDirSyncPerm(): void {
+unitTest({ perms: { read: false } }, function readDirSyncPerm() {
   assertThrows(() => {
     Deno.readDirSync("tests/");
   }, Deno.errors.PermissionDenied);
 });
 
-unitTest({ perms: { read: true } }, function readDirSyncNotDir(): void {
+unitTest({ perms: { read: true } }, function readDirSyncNotDir() {
   assertThrows(() => {
-    Deno.readDirSync("cli/tests/fixture.json");
+    Deno.readDirSync("cli/tests/testdata/fixture.json");
   }, Error);
 });
 
-unitTest({ perms: { read: true } }, function readDirSyncNotFound(): void {
+unitTest({ perms: { read: true } }, function readDirSyncNotFound() {
   assertThrows(() => {
     Deno.readDirSync("bad_dir_name");
   }, Deno.errors.NotFound);
 });
 
-unitTest({ perms: { read: true } }, async function readDirSuccess(): Promise<
-  void
-> {
+unitTest({ perms: { read: true } }, async function readDirSuccess() {
   const files = [];
-  for await (const dirEntry of Deno.readDir("cli/tests/")) {
+  for await (const dirEntry of Deno.readDir("cli/tests/testdata")) {
     files.push(dirEntry);
   }
   assertSameContent(files);
 });
 
-unitTest({ perms: { read: true } }, async function readDirWithUrl(): Promise<
-  void
-> {
+unitTest({ perms: { read: true } }, async function readDirWithUrl() {
   const files = [];
   for await (
-    const dirEntry of Deno.readDir(pathToAbsoluteFileUrl("cli/tests"))
+    const dirEntry of Deno.readDir(pathToAbsoluteFileUrl("cli/tests/testdata"))
   ) {
     files.push(dirEntry);
   }
   assertSameContent(files);
 });
 
-unitTest({ perms: { read: false } }, async function readDirPerm(): Promise<
-  void
-> {
+unitTest({ perms: { read: false } }, async function readDirPerm() {
   await assertThrowsAsync(async () => {
     await Deno.readDir("tests/")[Symbol.asyncIterator]().next();
   }, Deno.errors.PermissionDenied);
@@ -92,7 +88,7 @@ unitTest(
 
 unitTest(
   { perms: { read: true }, ignore: Deno.build.os == "windows" },
-  function readDirDevFdSync(): void {
+  function readDirDevFdSync() {
     for (const _ of Deno.readDirSync("/dev/fd")) {
       // We don't actually care whats in here; just that we don't panic on non regular file entries
     }
