@@ -8,8 +8,8 @@ import {
   unitTest,
 } from "./test_util.ts";
 
-unitTest({ perms: { read: true } }, function readFileSyncSuccess(): void {
-  const data = Deno.readFileSync("cli/tests/fixture.json");
+unitTest({ perms: { read: true } }, function readFileSyncSuccess() {
+  const data = Deno.readFileSync("cli/tests/testdata/fixture.json");
   assert(data.byteLength > 0);
   const decoder = new TextDecoder("utf-8");
   const json = decoder.decode(data);
@@ -17,9 +17,9 @@ unitTest({ perms: { read: true } }, function readFileSyncSuccess(): void {
   assertEquals(pkg.name, "deno");
 });
 
-unitTest({ perms: { read: true } }, function readFileSyncUrl(): void {
+unitTest({ perms: { read: true } }, function readFileSyncUrl() {
   const data = Deno.readFileSync(
-    pathToAbsoluteFileUrl("cli/tests/fixture.json"),
+    pathToAbsoluteFileUrl("cli/tests/testdata/fixture.json"),
   );
   assert(data.byteLength > 0);
   const decoder = new TextDecoder("utf-8");
@@ -28,23 +28,21 @@ unitTest({ perms: { read: true } }, function readFileSyncUrl(): void {
   assertEquals(pkg.name, "deno");
 });
 
-unitTest({ perms: { read: false } }, function readFileSyncPerm(): void {
+unitTest({ perms: { read: false } }, function readFileSyncPerm() {
   assertThrows(() => {
-    Deno.readFileSync("cli/tests/fixture.json");
+    Deno.readFileSync("cli/tests/testdata/fixture.json");
   }, Deno.errors.PermissionDenied);
 });
 
-unitTest({ perms: { read: true } }, function readFileSyncNotFound(): void {
+unitTest({ perms: { read: true } }, function readFileSyncNotFound() {
   assertThrows(() => {
     Deno.readFileSync("bad_filename");
   }, Deno.errors.NotFound);
 });
 
-unitTest({ perms: { read: true } }, async function readFileUrl(): Promise<
-  void
-> {
+unitTest({ perms: { read: true } }, async function readFileUrl() {
   const data = await Deno.readFile(
-    pathToAbsoluteFileUrl("cli/tests/fixture.json"),
+    pathToAbsoluteFileUrl("cli/tests/testdata/fixture.json"),
   );
   assert(data.byteLength > 0);
   const decoder = new TextDecoder("utf-8");
@@ -53,10 +51,8 @@ unitTest({ perms: { read: true } }, async function readFileUrl(): Promise<
   assertEquals(pkg.name, "deno");
 });
 
-unitTest({ perms: { read: true } }, async function readFileSuccess(): Promise<
-  void
-> {
-  const data = await Deno.readFile("cli/tests/fixture.json");
+unitTest({ perms: { read: true } }, async function readFileSuccess() {
+  const data = await Deno.readFile("cli/tests/testdata/fixture.json");
   assert(data.byteLength > 0);
   const decoder = new TextDecoder("utf-8");
   const json = decoder.decode(data);
@@ -64,23 +60,21 @@ unitTest({ perms: { read: true } }, async function readFileSuccess(): Promise<
   assertEquals(pkg.name, "deno");
 });
 
-unitTest({ perms: { read: false } }, async function readFilePerm(): Promise<
-  void
-> {
+unitTest({ perms: { read: false } }, async function readFilePerm() {
   await assertThrowsAsync(async () => {
-    await Deno.readFile("cli/tests/fixture.json");
+    await Deno.readFile("cli/tests/testdata/fixture.json");
   }, Deno.errors.PermissionDenied);
 });
 
-unitTest({ perms: { read: true } }, function readFileSyncLoop(): void {
+unitTest({ perms: { read: true } }, function readFileSyncLoop() {
   for (let i = 0; i < 256; i++) {
-    Deno.readFileSync("cli/tests/fixture.json");
+    Deno.readFileSync("cli/tests/testdata/fixture.json");
   }
 });
 
 unitTest(
   { perms: { read: true } },
-  async function readFileDoesNotLeakResources(): Promise<void> {
+  async function readFileDoesNotLeakResources() {
     const resourcesBefore = Deno.resources();
     await assertThrowsAsync(async () => await Deno.readFile("cli"));
     assertEquals(resourcesBefore, Deno.resources());
@@ -89,7 +83,7 @@ unitTest(
 
 unitTest(
   { perms: { read: true } },
-  function readFileSyncDoesNotLeakResources(): void {
+  function readFileSyncDoesNotLeakResources() {
     const resourcesBefore = Deno.resources();
     assertThrows(() => Deno.readFileSync("cli"));
     assertEquals(resourcesBefore, Deno.resources());
@@ -98,22 +92,26 @@ unitTest(
 
 unitTest(
   { perms: { read: true } },
-  async function readFileWithAbortSignal(): Promise<void> {
+  async function readFileWithAbortSignal() {
     const ac = new AbortController();
     queueMicrotask(() => ac.abort());
     await assertThrowsAsync(async () => {
-      await Deno.readFile("cli/tests/fixture.json", { signal: ac.signal });
+      await Deno.readFile("cli/tests/testdata/fixture.json", {
+        signal: ac.signal,
+      });
     });
   },
 );
 
 unitTest(
   { perms: { read: true } },
-  async function readTextileWithAbortSignal(): Promise<void> {
+  async function readTextileWithAbortSignal() {
     const ac = new AbortController();
     queueMicrotask(() => ac.abort());
     await assertThrowsAsync(async () => {
-      await Deno.readTextFile("cli/tests/fixture.json", { signal: ac.signal });
+      await Deno.readTextFile("cli/tests/testdata/fixture.json", {
+        signal: ac.signal,
+      });
     });
   },
 );
